@@ -14,7 +14,12 @@
     r2
 }
 
-## Use this with rda() results
+## rda. Function varpart() estimates adjusted R2 for partial model
+## rda(Y, X1, X2) as a difference of adjusted R2 values of models
+## rda(Y, cbind(X1,X2)) and rda(Y, X2), and we did the same here from
+## vegan release 2.0-3 to 2.4. However, I am not quite convinced that
+## this is correct, and therefore I disable adjusted R2 for partial
+## models while I study the issue (Jari Oksanen, 23/2/2018).
 `RsquareAdj.rda` <-
     function(x, ...)
 {
@@ -24,22 +29,25 @@
     if (is.null(x$pCCA)) {
         radj <- RsquareAdj(R2, n, m)
     } else {
-        ## Partial model: same adjusted R2 as for component [a] in two
-        ## source varpart model
-        R2p <- x$pCCA$tot.chi/x$tot.chi
-        p <- x$pCCA$rank
-        radj <- RsquareAdj(R2 + R2p, n, m + p) - RsquareAdj(R2p, n, p)
+        radj <- NA
     }
     list(r.squared = R2, adj.r.squared = radj)
 }
 
-## cca result: no RsquareAdj
-RsquareAdj.cca <-
-    function (x, permutations = 1000, ...) 
+## cca: for pCCA, see comment for rda. Earlier we used the same
+## equation both for pCCA and CCA which was inconsistent with rda. git
+## commit 9c25e6fdb4 has a version that was consistent with rda in
+## vegan 2.0-3 to 2.4-6.
+`RsquareAdj.cca` <-
+    function (x, permutations = 1000, ...)
 {
     r2 <- x$CCA$tot.chi / x$tot.chi
-    p <- permutest(x, permutations, ...)
-    radj <- 1 - ((1 - r2) / (1 - mean(p$num / x$tot.chi)))
+    if (is.null(x$pCCA) || x$pCCA$rank == 0) {
+        p <- permutest(x, permutations, ...)
+        radj <- 1 - ((1 - r2) / (1 - mean(p$num) / x$tot.chi))
+    } else {
+        radj <- NA
+    }
     list(r.squared = r2, adj.r.squared = radj)
 }
 
