@@ -1,15 +1,16 @@
 hiersimu.default <-
 function(y, x, FUN, location = c("mean", "median"),
-relative = FALSE, drop.highest = FALSE, nsimul=99, ...)
+         relative = FALSE, drop.highest = FALSE, nsimul=99,
+         method = "r2dtable", ...)
 {
     ## evaluate formula
     lhs <- as.matrix(y)
     if (missing(x))
-        x <- cbind(level_1=seq_len(nrow(lhs)), 
+        x <- cbind(level_1=seq_len(nrow(lhs)),
             leve_2=rep(1, nrow(lhs)))
     rhs <- data.frame(x)
     rhs[] <- lapply(rhs, as.factor)
-    rhs[] <- lapply(rhs, droplevels)
+    rhs[] <- lapply(rhs, droplevels, exclude = NA)
     nlevs <- ncol(rhs)
     if (is.null(colnames(rhs)))
         colnames(rhs) <- paste("level", 1:nlevs, sep="_")
@@ -32,7 +33,7 @@ relative = FALSE, drop.highest = FALSE, nsimul=99, ...)
     rval <- as.data.frame(rval[rev(1:length(rval))])
     l2 <- sapply(rval, function(z) length(unique(z)))
     if (any(l1 != l2))
-        warning("levels are not perfectly nested")
+        stop("levels are not perfectly nested")
 
     ## aggregate response matrix
     fullgamma <-if (nlevels(rhs[,nlevs]) == 1)
@@ -40,15 +41,13 @@ relative = FALSE, drop.highest = FALSE, nsimul=99, ...)
     if (fullgamma && drop.highest)
         nlevs <- nlevs - 1
     if (nlevs == 1 && relative)
-        stop("'relative=FALSE' makes no sense with 1 level")
+        stop("'relative=FALSE' makes no sense with one level")
     ftmp <- vector("list", nlevs)
     for (i in 1:nlevs) {
         ftmp[[i]] <- as.formula(paste("~", tlab[i], "- 1"))
     }
 
-    ## is there a method/burnin/thin in ... ?
-    method <- if (is.null(list(...)$method))
-        "r2dtable" else list(...)$method
+    ## is there burnin/thin in ... ?
     burnin <- if (is.null(list(...)$burnin))
         0 else list(...)$burnin
     thin <- if (is.null(list(...)$thin))
