@@ -1,9 +1,11 @@
-"scores.metaMDS" <-
+`scores.metaMDS` <-
     function(x, display = c("sites", "species"), shrink = FALSE, choices, ...)
 {
     display <- match.arg(display)
     if (missing(choices))
-        choices <- 1:x$dims
+        choices <- 1:x$ndim
+    else
+        choices <- choices[choices <= x$ndim]
     if (display == "sites")
         X <- x$points
     else if (display == "species") {
@@ -11,7 +13,7 @@
         if (shrink) {
             mul <- sqrt(attr(X, "shrinkage"))
             if (is.null(mul))
-                warning("Species cannot be shrunken, because they were not expanded")
+                warning("species cannot be shrunken, because they were not expanded")
             else {
                 cnt <- attr(X, "centre")
                 X <- sweep(X, 2, cnt, "-")
@@ -21,6 +23,6 @@
         }
     }
     colnames(X) <- paste("NMDS", 1:ncol(X), sep="")
-    X[, choices]
+    X[, choices, drop = FALSE]
 }
 
